package de.ozgcloud.common.grpc;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import io.grpc.Metadata;

class GrpcUtilTest {
	private static final String KEY_VALUE_BIN = "key_value-bin";
	private static final String KEY_VALUE_STRING = "key_value";
	private static final String UNKNOWN_KEY_BIN = "unknown_key-bin";

	@DisplayName("Test Grpc Utility methods")
	@Nested
	class TestGrpcUtils {

		@DisplayName("Test creation of MetaData keys")
		@Nested
		class TestKeyGeneration {

			@Test
			void shouldCreateBinaryKey() {
				var key = GrpcUtil.createKeyOf(KEY_VALUE_BIN);

				assertThat(key.name()).isEqualTo(KEY_VALUE_BIN);
			}

			@Test
			void shouldThrowExceptionBecauseOfMissingBinSuffix() {
				assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(() -> GrpcUtil.createKeyOf("key"));
			}

			@Test
			void shouldCreateAsciiKey() {
				var key = GrpcUtil.keyOfString(KEY_VALUE_STRING);

				assertThat(key.name()).isEqualTo(KEY_VALUE_STRING);
			}

			@Test
			void shouldThrowExceptionBecauseOfBinSuffix() {
				assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(() -> GrpcUtil.keyOfString(KEY_VALUE_BIN));
			}
		}

		@DisplayName("Test reading values from the grpc header")
		@Nested
		class TestReadingFromHeader {
			private static final String TEST_VALUE = "test-value";

			private Metadata header = new Metadata();

			@BeforeEach
			void initMetadata() {
				header.put(GrpcUtil.createKeyOf(KEY_VALUE_BIN), TEST_VALUE.getBytes());
			}

			@Test
			void shouldReadFromHeader() {
				var value = GrpcUtil.getFromHeaders(KEY_VALUE_BIN, header);

				assertThat(value).isEqualTo(TEST_VALUE);
			}

			@Test
			void shouldReturnNullOnUnknownKey() {
				var value = GrpcUtil.getFromHeaders(UNKNOWN_KEY_BIN, header);

				assertThat(value).isNull();
			}
		}

		@DisplayName("Test reading multiple values of a key from the grpc header")
		@Nested
		class TestReadingCollection {
			private static final String TEST_VALUE_1 = "test-value-1";
			private static final String TEST_VALUE_2 = "test-value-2";

			private Metadata header = new Metadata();

			@BeforeEach
			void initMetadata() {
				header.put(GrpcUtil.createKeyOf(KEY_VALUE_BIN), TEST_VALUE_1.getBytes());
				header.put(GrpcUtil.createKeyOf(KEY_VALUE_BIN), TEST_VALUE_2.getBytes());
			}

			@Test
			void shouldReadFromHeader() {
				var values = GrpcUtil.getCollection(KEY_VALUE_BIN, header);

				assertThat(values).hasSize(2).contains(TEST_VALUE_1, TEST_VALUE_2);
			}

			@Test
			void shouldGetEmptyCollectionOnUnknownKey() {
				var values = GrpcUtil.getCollection(UNKNOWN_KEY_BIN, header);

				assertThat(values).isEmpty();
			}
		}
	}
}
